#!/usr/local/bin/perl -w

###########################################################################
#
# LAST UPDATED: Jun17, 2003 
# sval2parser.pl version 0.1
#
###########################################################################
#
# This program checks if the SOURCE file has certain xml tags conforming 
# with the format of xml tags as specified in the Senseval-2 data format.
# Specifically, the program makes one pass at the SOURCE file and it 
# looks for the relevant xml tags in the order specified below:
#
# ONE
# The first tag looked for is the <corpus lang='.*'> tag. The text may have 
# lines consisting of single xml tags before the corpus tag. If it has 
# multiple xml tags or non-xml tokens or if the corpus tag is invalid or
# missing, the program stops execution at that point and specifies the
# error.  
# TWO
# Once a valid corpus tag is found the program continues reading line by 
# line. These lines should be either blank or contain a single xml tag,
# until the <context> tag is found. Additionally, the tags should come
# in the order - <lexelt item=".*">, <instance id=".*".*>, <answer 
# instance="instance ID from previous tag" senseid=".*"/> (if training 
# file) and <context>.
# THREE
# Within the context, the <head> ... </head> tag pair must exist and in 
# that order. Check is made to see if the satelite tag exists and if so
# in proper format and order. The end of context is identified by a valid
# </context> tag. It should exist on a new line with no other tags/tokens
# FOUR
# The lines to follow must also contain a single valid xml tag or be 
# blank. The tags must be in the order - </instance>, (optionally) 
# </lexelt item>.
# FIVE
# If a valid </corpus> tag is not found next, steps TWO, THREE, FOUR and
# FIVE are repeated. It may be noted that the <lexelt item=".*"> tag is
# not looked for if the </instance> tag was not found in step FOUR.
#
# In case a valid tag was not found or a tag was found out of order, 
# the program stops executions and prints information about the anomaly 
# to the standard output. 
#
# USAGE :  sval2parser.pl [OPTIONS] SOURCE
#
#          OPTIONS
#		--test		SOURCE is a test file.
#				Assumed to be training otherwise.
#		--version       Prints the version number.
#       	--help          Prints help message.
#
# It may be noted that multiple valid answer tags for an instance are 
# allowed in case of a training file.
#
###########################################################################
#
# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###########################################################################

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("test", "version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
if ( defined $opt_test )    { $test = 1; $opt_test = 1; }
else                        { $test = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 0 ) {
    &usage();
    exit;}

$inpfl = shift();

# Open the various files
open(INP,"<$inpfl");

# Read lines from input file
while($ln = <INP>) {
    $lnno++;
    if($ln =~ /^\s*$/) { next; }
    if($ln =~ /^\s*<corpus\s+lang\s*=\s*[\"'][^\"']*[\"']>\s*$/) { last; }	
    # Ignore lines with just an xml tag which is not the corpus tag and 
    # occurring before the <corpus tag
    if($ln =~ /^\s*<[^>]*>\s*$/) { next; }
    print "No <corpus lang=\".*\"> tag\n";
    exit(); }

# A simple Automaton
$state = 0;
$lexelt = 0;
$instID = "No Instance ID yet";
# Read lines from input file
while($ln = <INP>) {
    $lnno++;
    if($ln =~ /^\s*$/) { next; }
    SWITCH: {
	 if ($state == 0) { if ($ln =~ /^\s*<\/corpus>\s*$/) { $state = 1000; last; } 

			    if ($ln =~ /^\s*<lexelt\s+item\s*=\s*\"([^\"]*)\">\s*$/) { 
                                   $state = 1; $lexelt = $1; last SWITCH; } 
			    print "\nNo valid lexelt item OR invalid </corpus> tag after instance ID: $instID\nStopped at line: $lnno\n\n";
			    $state = 100; last SWITCH; } 

	 if ($state == 1) { if ($ln =~ /^\s*<\/lexelt>\s*$/) { $state = 0; last SWITCH; }

			    if ($ln =~ /^\s*<instance\s+id\s*=\s*\"([^\"]*)\"[^>]*>\s*$/) { 
				   if($test) 	{ $state = 3; }
				   else		{ $state = 2; }
				   $instID = $1; last SWITCH; } 
			    print "\nNo valid instance ID OR invalid </lexelt> tag after instance ID: $instID\nStopped at line: $lnno\n\n";
			    $state = 100; last SWITCH; }

	 if ($state == 2) { if ($ln =~ /^\s*<answer\s+instance\s*=\s*\"$instID"\s+senseid\s*=\s*\"([^\"]*)\"\/>\s*$/) 
				   { $state = 3; $senseID = $1; last SWITCH; } 
			    print "\nNo valid answer ID tag after instance ID: $instID\nStopped at line: $lnno\n\n";
                            $state = 100; last SWITCH; }

	 if ($state == 3) { if ($ln =~ /^\s*<context>\s*$/) { $state = 4; last SWITCH; }
			    # CHANGE Jun17
			    if ($ln =~ /^\s*<answer\s+instance\s*=\s*\"$instID"\s+senseid\s*=\s*\"([^\"]*)\"\/>\s*$/)
				   { $state = 3; $senseID = $1; last SWITCH; }
			    print "\nNo valid <context> tag after instance ID: $instID\nStopped at line: $lnno\n\n";
			    $state = 100; last SWITCH; }

         if ($state == 4) { if ($ln =~ /<\/?(lexelt|instance|answer|context|corpus)/) { 
				   print "\n$1 tag out of place OR invalid/missing head tag pair.\n";
				   print "Stopped after instance ID: $instID at line: $lnno\n\n";
				   $state = 100; last SWITCH; }
			    # Check if sat tag present
			    if ($ln =~ /<\/?sat/) { 
				   # Check for valid sat tag pair
				   if ($ln !~ /<sat[^>]*>.+<\/sat>/) {
					 print "\nInvalid sat tag pair. \nStopped at line: $lnno\n\n";
					 $state = 100; last SWITCH; } }

			    if ($ln =~ /<head[^>]*>.+<\/head>/) { $state = 5; last SWITCH; } }

	 if ($state == 5) { if ($ln =~ /<(context|\/?(head|sat|lexelt|instance|corpus|answer))/) { 
				   print "\n<$1 tag out of place OR invalid/missing </context> tag.\n";
				   print "Stopped after instance ID: $instID at line: $lnno\n\n";
				   $state = 100; last SWITCH; }
                            if ($ln =~ /^\s*<\/context>\s*$/) { $state = 6; last SWITCH; } }

	 if ($state == 6) { if ($ln =~ /^\s*<\/instance>\s*$/) { $state = 1; last SWITCH; } 
                                   print "\nNo valid </instance> tag after instance ID: $instID\nStopped after line: $lnno\n\n";
                                   $state = 100; last SWITCH; }

	 if ($state == 100) { print "ERROR IN DATA FORMAT\n\n"; exit(); } 

    } # END SWITCH
    if($state == 1000) { last;}
} # END WHILE

if($state == 1000) {
    while(<INP>) {
	$lnno++;
	if(/^\s*$/) { next; }
	print "\nNon blank lines after </corpus> tag.\n";
	print "Stopped at line: $lnno\n\n";
	exit();
    }
}
else {
    print "\nEnd of file without valid </corpus> tag\n\n";
    exit();
}

$senseID = "";

print"\nDATA CONFORMS TO SENSEVAL-2 DATA FORMAT\n\n";

# Subroutine to print help message
sub help{
    print "Usage: sval2parser.pl [OPTIONS] SOURCE\n\n";
    print "This program checks if the SOURCE file has certain xml tags conforming with the format of xml tags as specified in the Senseval-2 data format. Specifically, the program makes one pass at the SOURCE file and looks for valid xml tags and in proper order.\n\n";
    print "OPTIONS:\n\n";
    print "  --test             SOURCE is a test file.\n";
    print "                     Assumed to be training otherwise.\n";
    print "  --version          Prints the version number.\n";
    print "  --help             Prints the help message.\n\n";}

sub usage{
    print "Usage: sval2parser.pl [OPTIONS] SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type sval2parser.pl --help for help.\n";}

# function to output the version number
sub showVersion
{
    print STDERR "sval2parser.pl      -        version 0.1\n";
    print STDERR "A component of the Sval2Check 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}
